/*
 * statemachine.cpp
 *
 *  Created on: 21 Sep 2023
 *      Author: Dario
 */

#include <sta/tacos/statemachine.hpp>

#include <sta/debug/debug.hpp>


namespace sta
{
	namespace tacos
	{
		Statemachine::Statemachine()
			: currentState_{STA_TACOS_INITIAL_STATE},
			  lockoutTimer_{[](void *){}, nullptr},
			  failsafeTimer_{[](void *){}, nullptr}
		{
			STA_ASSERT(STA_TACOS_INITIAL_STATE < STA_TACOS_NUM_STATES);
		}

		void Statemachine::init()
		{
			STA_DEBUG_PRINTLN("INITIALIZING STATEMACHINE");

			lockoutTimer_.setCallback([](void *) { STA_DEBUG_PRINTLN("[LOCKOUT TIMEOUT]"); }, nullptr);
			lockoutTimer_.start(5000);

			failsafeTimer_.setCallback([](void *) { STA_DEBUG_PRINTLN("[FAILSAFE TIMEOUT]"); }, nullptr);
			failsafeTimer_.start(10000);
		}

		void Statemachine::func()
		{
			STA_DEBUG_PRINTLN("LOOPY LOOP IN STATEMACHINE.");

			osDelay(3000);
		}

		uint16_t Statemachine::getCurrentState() const
		{
			return currentState_;
		}

		Statemachine* Statemachine::_instance = nullptr;

	}  // namespace tacos
}  // namespace sta



